-- Only load this addon is the character is a Druid
if ( select(2, UnitClass("player")) ~= "DRUID" ) then return end

EclipseBarPlus = LibStub("AceAddon-3.0"):NewAddon("EclipseBarPlus", "AceConsole-3.0", "AceEvent-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("EclipseBarPlus")

local defaults = {
		profile =  {
		scale = 1,
		font_size = 12,
		show_power = true,
		show_percentage = false,
		negative_power = false,
		locked = false,
		hide_blizzard_bar = true,
		position = {
			POINT = "CENTER",
			RELATIVE = "CENTER",
			XCOORD = 0,
			YCOORD = 1
		},
    },
}

local options = {
	name = "Eclipse Bar Plus",
	handler = EclipseBarPlus,
	type = "group",
	args = {
		locked = {
			type = "toggle",
			name = L["Lock"],
			desc = L["Locks the eclipse bar, preventing it from moving"],
			set = function(info, value)
				-- Save variable
				EclipseBarPlus.db.profile.locked = value
			
				-- Apply to frame
				if value then
					EclipseBarPlusFrame:EnableMouse(false)
				else
					EclipseBarPlusFrame:EnableMouse(true)
				end
			end,
			get = function(info)
				return EclipseBarPlus.db.profile.locked
			end,
			width = "full",
			order = 10,
		},
		hide_blizzard_bar = {
			type = "toggle",
			name = L["Hide Blizzard Eclipse Bar"],
			desc = L["Hides the original Eclipse Bar from Blizzard"],
			set = function(info, value)
				-- Save variable
				EclipseBarPlus.db.profile.hide_blizzard_bar = value
			
				-- Apply to frame
				if value then
					EclipseBarFrame:Hide();
				else
					EclipseBarFrame:Show();
				end
			end,
			get = function(info)
				return EclipseBarPlus.db.profile.hide_blizzard_bar
			end,
			width = "full",
			order = 20,
		},
		scale = {
			type = "range",
			name = L["Scale"],
			desc = L["Scale of the frame"],
			set = function(info, value)
				-- Save variable
				EclipseBarPlus.db.profile.scale = value
				
				-- Apply the new scale
				EclipseBarPlusFrame:SetScale(value)
			end,
			get = function(info)
				return EclipseBarPlus.db.profile.scale
			end,
			min = 0,
			max = 3,
			step = 0.1,
			order = 15,
			order = 30,
		},
		power_group = {
			type = "group",
			name = L["Power text options"],
			inline = true,
			order = 50,
			args = {
				show_power = {
					type = "toggle",
					name = L["Show power text"],
					desc = L["Wether or not to display the power text on the eclipse bar"],
					set = function(info, value)
						-- Save variable
						EclipseBarPlus.db.profile.show_power = value
			
						-- Apply to the power text
						if value == true then
							EclipseBarPlusFrame.powerText:Show()
						else
							EclipseBarPlusFrame.powerText:Hide()
						end
					end,
					get = function(info)
						return EclipseBarPlus.db.profile.show_power
					end,
					width = "full",
					order = 10,
				},
				negative_power = {
					type = "toggle",
					name = L["Display Lunar power as a negative number"],
					desc = L["Display Lunar power as a negative number. So the power on the bar is negative when approaching lunar power and becomes positive when approaching solar power."],
					set = function(info, value)
						-- Save variable
						EclipseBarPlus.db.profile.negative_power = value
					end,
					get = function(info)
						return EclipseBarPlus.db.profile.negative_power
					end,
					width = "full",
					disabled = function() return not EclipseBarPlus.db.profile.show_power end,
					order = 20,
				},
				show_percentage = {
					type = "toggle",
					name = L["Show power text as percentage"],
					desc = L["Shows power text as a percentage instead of an absolute number."],
					set = function(info, value)
						-- Save variable
						EclipseBarPlus.db.profile.show_percentage = value
					end,
					get = function(info)
						return EclipseBarPlus.db.profile.show_percentage
					end,
					width = "full",
					disabled = function() return not EclipseBarPlus.db.profile.show_power end,
					order = 30,
				},
				font_size = {
					type = "range",
					name = L["Scale the size of the font"],
					desc = L["Change the font size of the power indicator."],
					set = function(info, value)
						-- Save variable
						EclipseBarPlus.db.profile.font_size = value
						
						-- Apply the new scale
						EclipseBarPlusFrame.powerText:SetTextHeight(value)
						EclipseBarPlusFrame.powerText:SetPoint("CENTER","EclipseBarPlusFrame",0,0)
					end,
					get = function(info)
						return EclipseBarPlus.db.profile.font_size
					end,
					min = 2,
					max = 40,
					step = 1,
					order = 40,
					disabled = function() return not EclipseBarPlus.db.profile.show_power end,
				},
			},
		},
	},
}

function EclipseBarPlus:OnInitialize()
	-- Called when the addon is loaded
	self.db = LibStub("AceDB-3.0"):New("EclipseBarPlusDB", defaults, "Default")

	LibStub("AceConfig-3.0"):RegisterOptionsTable("EclipseBarPlusDB", options)
	self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("EclipseBarPlusDB", "Eclipse Bar Plus")
	self:RegisterChatCommand("ebp", "ChatCommand")
	self:RegisterChatCommand("eclipsebarplus", "ChatCommand")
	
	-- Set the position
	local pos = EclipseBarPlus.db.profile.position
	EclipseBarPlusFrame:SetMovable()
	EclipseBarPlusFrame:ClearAllPoints()
	EclipseBarPlusFrame:SetPoint(pos["POINT"], "UIParent", pos["RELATIVE"], pos["XCOORD"], pos["YCOORD"])
	
	-- Set the scale
	EclipseBarPlusFrame:SetScale(EclipseBarPlus.db.profile.scale)
	
	-- Set the font scale
	EclipseBarPlusFrame.powerText:SetTextHeight(EclipseBarPlus.db.profile.font_size)
	
	-- Set the lock
	if EclipseBarPlus.db.profile.locked then
		EclipseBarPlusFrame:EnableMouse(false)
	else
		EclipseBarPlusFrame:EnableMouse(true)
	end
end

function EclipseBarPlus:OnDragStart()
	EclipseBarPlusFrame:StartMoving()
end

function EclipseBarPlus:OnDragStop()
	EclipseBarPlusFrame:StopMovingOrSizing()
	local pos = {}
	pos["POINT"], _, pos["RELATIVE"], pos["XCOORD"], pos["YCOORD"] = EclipseBarPlusFrame:GetPoint()
	EclipseBarPlus.db.profile.position = pos
end

function EclipseBarPlus:ChatCommand(input)
	-- Open dialog if no parameters, else do something with the parameters
	if not input or input:trim() == "" then
		InterfaceOptionsFrame_OpenToCategory(self.optionsFrame)
	else
		LibStub("AceConfigCmd-3.0").HandleCommand(SpamageMeters, "ebp", "EclipseBarPlus", input)
	end
end


ECLIPSE_BAR_POWER_INDEX = 8;
MOONKIN_FORM = 31


ECLIPSE_BAR_TRAVEL = 38;

ECLIPSE_BAR_SOLAR_BUFF_ID = 48517;
ECLIPSE_BAR_LUNAR_BUFF_ID = 48518;



ECLIPSE_ICONS =  {};
ECLIPSE_ICONS["moon"] = { 
												norm = { x=23, y=23, left=0.55859375, right=0.64843750, top=0.57031250, bottom=0.75000000 } ,
												dark  = { x=23, y=23, left=0.55859375, right=0.64843750, top=0.37500000, bottom=0.55468750 } ,
												big    = { x=43, y=45, left=0.73437500, right=0.90234375, top=0.00781250, bottom=0.35937500 } ,
											}
ECLIPSE_ICONS["sun"] = { 
												norm = { x=23, y=23, left=0.65625000, right=0.74609375, top=0.37500000, bottom=0.55468750 } ,
												dark  = { x=23, y=23, left=0.55859375, right=0.64843750, top=0.76562500, bottom=0.94531250 } ,
												big    = { x=43, y=45, left=0.55859375, right=0.72656250, top=0.00781250, bottom=0.35937500 } ,
											}


function EclipseBarPlus_UpdateShown(self)
	if VehicleMenuBar:IsShown() then
		return;
	end

	-- Disable rune frame if not a DRUID.
	local _, class = UnitClass("player");
	local form  = GetShapeshiftFormID();

	if  class == "DRUID" and (form == MOONKIN_FORM or not form) then
		if GetMasteryIndex(GetActiveTalentGroup(false, false)) == 1 then
--			self.showPercent = GetCVarBool("statusTextPercentage");
--			if GetCVarBool("playerStatusText") then
			if EclipseBarPlus.db.profile.show_power then
				self.powerText:Show();
				self.lockShow = true;
			else
				self.powerText:Hide();
				self.lockShow = false;
			end
			self:Show();
			if EclipseBarPlus.db.profile.hide_blizzard_bar then
				EclipseBarFrame:Hide(); -- Hide Blizzard Eclipse Bar
			end
			
		else
			self:Hide();
		end
	else
		self:Hide();
		return;
	end
end

function EclipseBarPlus_Update(self)
	local power = UnitPower( PlayerFrame.unit, ECLIPSE_BAR_POWER_INDEX );
	local maxPower = UnitPowerMax( PlayerFrame.unit, ECLIPSE_BAR_POWER_INDEX );
	if EclipseBarPlus.db.profile.show_percentage then
		if EclipseBarPlus.db.profile.negative_power then
			self.powerText:SetText((power/maxPower*100).."%");
		else
			self.powerText:SetText(abs(power/maxPower*100).."%");
		end
	else
		if EclipseBarPlus.db.profile.negative_power then
			self.powerText:SetText(power);
		else
			self.powerText:SetText(abs(power));
		end
	end

	local xpos =  ECLIPSE_BAR_TRAVEL*(power/maxPower)
	self.marker:SetPoint("CENTER", xpos, 0);


	if power < 0 and self.side ~= "moon" then
		self.glow:ClearAllPoints();
		local glowInfo = ECLIPSE_ICONS["moon"].big;
		self.glow:SetPoint("CENTER", self.moon, "CENTER", 0, 0);
		self.glow:SetWidth(glowInfo.x);
		self.glow:SetHeight(glowInfo.y);
		self.glow:SetTexCoord(glowInfo.left, glowInfo.right, glowInfo.top, glowInfo.bottom);
		 self.side = "moon";
	elseif power > 0 and  self.side ~= "sun" then
		self.glow:ClearAllPoints();
		local glowInfo = ECLIPSE_ICONS["sun"].big;
		self.glow:SetPoint("CENTER", self.sun, "CENTER", 0, 0);
		self.glow:SetWidth(glowInfo.x);
		self.glow:SetHeight(glowInfo.y);
		self.glow:SetTexCoord(glowInfo.left, glowInfo.right, glowInfo.top, glowInfo.bottom);
		self.side = "sun";
	elseif power == 0  then
		self.side = "none";
	end
end





function EclipseBarPlus_OnLoad (self)
	self:RegisterEvent("UPDATE_SHAPESHIFT_FORM");
	self:RegisterEvent("PLAYER_TALENT_UPDATE");	
	self:RegisterEvent("MASTERY_UPDATE");
	self:RegisterEvent("CVAR_UPDATE");	
	self:RegisterEvent("UNIT_AURA");
	
	self:RegisterForDrag("LeftButton");

	self.lockShow = false;

	self.sunBar:SetAlpha(0);
	self.moonBar:SetAlpha(0);
	self.darkSun:SetAlpha(0);
	self.darkMoon:SetAlpha(0);
	self.glow:SetAlpha(0);
end


function EclipseBarPlus_CheckBuffs (self) 
	local hasLunarEclipse = false;
	local hasSolarEclipse = false;

	local unit = PlayerFrame.unit;
	local j = 1;
	local name, _, _, _, _, _, _, _, _, _, spellID = UnitBuff(unit, j);
	while name do 
		if spellID == ECLIPSE_BAR_SOLAR_BUFF_ID then
			hasSolarEclipse = true;
		elseif spellID == ECLIPSE_BAR_LUNAR_BUFF_ID then
			hasLunarEclipse = true;
		end
		j=j+1;
		name, _, _, _, _, _, _, _, _, _, spellID = UnitBuff(unit, j);
	end

	if hasLunarEclipse then
		if self.moonDeactivate:IsPlaying() then
			self.moonDeactivate:Stop();
		elseif not self.moonActivate:IsPlaying() and self.moonBar:GetAlpha() == 0 then
			self.moonActivate:Play();
		end
	else
		if self.moonActivate:IsPlaying() then
			self.moonActivate:Stop();
		elseif not self.moonDeactivate:IsPlaying() and self.moonBar:GetAlpha() == 1 then
			self.moonDeactivate:Play();
		end
	end

	if hasSolarEclipse then
		if self.sunDeactivate:IsPlaying() then
			self.sunDeactivate:Stop();
		elseif not self.sunActivate:IsPlaying() and self.sunBar:GetAlpha() == 0 then
			self.sunActivate:Play();
		end
	else
		if self.sunActivate:IsPlaying() then
			self.sunActivate:Stop();
		elseif not self.sunDeactivate:IsPlaying() and self.sunBar:GetAlpha() == 1 then
			self.sunDeactivate:Play();
		end
	end
end 



function EclipseBarPlus_OnEvent (self, event, ...)
	if event == "UNIT_AURA" then
		local arg1 = ...;
		if arg1 ==  PlayerFrame.unit then
			EclipseBarPlus_CheckBuffs(self);
		end
	else
		EclipseBarPlus_UpdateShown(self);
	end
end